@@ -12,6 +12,7 @@
 #include <ReviewsBackend/Rating.h>
 #include <ReviewsBackend/Review.h>
 
+#include <qnumeric.h>
 #include <resources/AbstractResource.h>
 #include <resources/AbstractResourcesBackend.h>
 
@@ -31,28 +32,25 @@
 #include <QStandardPaths>
 
 #include <QFutureWatcher>
-#include <QNetworkConfiguration>
-#include <QNetworkConfigurationManager>
 #include <QtConcurrentRun>
 
 // #define APIURL "http://127.0.0.1:5000/1.0/reviews/api"
 #define APIURL "https://odrs.gnome.org/1.0/reviews/api"
 
+QSharedPointer<OdrsReviewsBackend> OdrsReviewsBackend::global()
+{
+    static QSharedPointer<OdrsReviewsBackend> var = nullptr;
+    if (!var) {
+        var = QSharedPointer<OdrsReviewsBackend>(new OdrsReviewsBackend());
+    }
+
+    return var;
+}
+
 OdrsReviewsBackend::OdrsReviewsBackend()
     : AbstractReviewsBackend(nullptr)
 {
     fetchRatings();
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    QNetworkConfigurationManager *manager = new QNetworkConfigurationManager(this);
-
-    connect(manager, &QNetworkConfigurationManager::onlineStateChanged, this, [this](bool online) {
-        if (online && !m_errorMessage.isEmpty()) {
-            m_errorMessage.clear();
-            Q_EMIT errorMessageChanged();
-            fetchRatings();
-        }
-    });
-#endif
 }
 
 OdrsReviewsBackend::~OdrsReviewsBackend() noexcept
@@ -80,7 +78,7 @@ void OdrsReviewsBackend::fetchRatings()
         fetchRatings = true;
     }
 
-    qDebug() << "fetch ratings!" << fetchRatings;
+    qCWarning(LIBDISCOVER_LOG) << "OdrsReviewsBackend: Fetch ratings:" << fetchRatings;
     if (fetchRatings) {
         setFetching(true);
         KIO::FileCopyJob *getJob = KIO::file_copy(ratingsUrl, fileUrl, -1, KIO::Overwrite | KIO::HideProgressInfo);
@@ -103,7 +101,7 @@ void OdrsReviewsBackend::ratingsFetched(KJob *job)
 {
     setFetching(false);
     if (job->error()) {
-        qCWarning(LIBDISCOVER_LOG) << "Failed to fetch ratings " << job->errorString();
+        qCWarning(LIBDISCOVER_LOG) << "OdrsReviewsBackend: Failed to fetch ratings:" << job->errorString();
     } else {
         parseRatings();
     }
@@ -137,9 +135,9 @@ void OdrsReviewsBackend::fetchReviews(AbstractResource *app, int page)
         return;
     }
     Q_UNUSED(page)
-    const QString version = app->isInstalled() ? app->installedVersion() : app->availableVersion();
+    QString version = app->isInstalled() ? app->installedVersion() : app->availableVersion();
     if (version.isEmpty()) {
-        return;
+        version = QStringLiteral("unknown");
     }
     setFetching(true);
 
@@ -170,8 +168,8 @@ void OdrsReviewsBackend::reviewsFetched()
     const QByteArray data = reply->readAll();
     const auto networkError = reply->error();
     if (networkError != QNetworkReply::NoError) {
-        qCWarning(LIBDISCOVER_LOG) << "error fetching reviews:" << reply->errorString() << data;
-        m_errorMessage = i18n("Error while fetching reviews: %1", reply->errorString());
+        qCWarning(LIBDISCOVER_LOG) << "OdrsReviewsBackend: Error fetching reviews:" << reply->errorString() << data;
+        m_errorMessage = i18n("Technical error message: %1", reply->errorString());
         Q_EMIT errorMessageChanged();
         setFetching(false);
         return;
@@ -180,7 +178,7 @@ void OdrsReviewsBackend::reviewsFetched()
     QJsonParseError error;
     const QJsonDocument document = QJsonDocument::fromJson(data, &error);
     if (error.error) {
-        qWarning() << "odrs: error parsing reviews" << reply->url() << error.errorString();
+        qWarning() << "OdrsReviewsBackend: Error parsing reviews:" << reply->url() << error.errorString();
     }
 
     AbstractResource *resource = qobject_cast<AbstractResource *>(reply->request().originatingObject());
@@ -220,9 +218,9 @@ void OdrsReviewsBackend::usefulnessSubmitted()
     QNetworkReply *reply = qobject_cast<QNetworkReply *>(sender());
     const auto networkError = reply->error();
     if (networkError == QNetworkReply::NoError) {
-        qCWarning(LIBDISCOVER_LOG) << "Usefulness submitted";
+        qCWarning(LIBDISCOVER_LOG) << "OdrsReviewsBackend: Usefulness submitted";
     } else {
-        qCWarning(LIBDISCOVER_LOG) << "Failed to submit usefulness: " << reply->errorString();
+        qCWarning(LIBDISCOVER_LOG) << "OdrsReviewsBackend: Failed to submit usefulness: " << reply->errorString();
         Q_EMIT error(i18n("Error while submitting usefulness: %1", reply->errorString()));
     }
     reply->deleteLater();
@@ -272,16 +270,16 @@ void OdrsReviewsBackend::reviewSubmitted(QNetworkReply *reply)
     if (networkError == QNetworkReply::NoError) {
         AbstractResource *resource = qobject_cast<AbstractResource *>(reply->request().originatingObject());
         Q_ASSERT(resource);
-        qCWarning(LIBDISCOVER_LOG) << "Review submitted" << resource;
+        qCWarning(LIBDISCOVER_LOG) << "OdrsReviewsBackend: Review submitted for" << resource;
         if (resource) {
             const QJsonDocument document({resource->getMetadata(QStringLiteral("ODRS::review_map")).toObject()});
             parseReviews(document, resource);
         } else {
-            qCWarning(LIBDISCOVER_LOG) << "Failed to submit review: missing object";
+            qCWarning(LIBDISCOVER_LOG) << "OdrsReviewsBackend: Failed to submit review: missing object";
         }
     } else {
         Q_EMIT error(i18n("Error while submitting review: %1", reply->errorString()));
-        qCWarning(LIBDISCOVER_LOG) << "Failed to submit review: " << reply->errorString();
+        qCWarning(LIBDISCOVER_LOG) << "OdrsReviewsBackend: Failed to submit review:" << reply->errorString();
     }
     reply->deleteLater();
 }
@@ -329,14 +327,14 @@ void OdrsReviewsBackend::parseRatings()
     fw->setFuture(QtConcurrent::run([] {
         QFile ratingsDocument(QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + QStringLiteral("/ratings/ratings"));
         if (!ratingsDocument.open(QIODevice::ReadOnly)) {
-            qWarning() << "odrs: Could not open file" << ratingsDocument.fileName();
+            qWarning() << "OdrsReviewsBackend: Could not open file" << ratingsDocument.fileName();
             return QJsonDocument::fromJson({});
         }
 
         QJsonParseError error;
         const auto ret = QJsonDocument::fromJson(ratingsDocument.readAll(), &error);
         if (error.error) {
-            qWarning() << "odrs: error parsing ratings" << ratingsDocument.errorString() << error.errorString();
+            qWarning() << "OdrsReviewsBackend: Error parsing ratings: " << ratingsDocument.errorString() << error.errorString();
         }
         return ret;
     }));
@@ -356,10 +354,48 @@ void OdrsReviewsBackend::parseReviews(const QJsonDocument &document, AbstractRes
         for (auto it = reviews.begin(); it != reviews.end(); it++) {
             const QJsonObject review = it->toObject();
             if (!review.isEmpty()) {
+                // Same ranking algorythm Gnome Software uses
                 const int usefulFavorable = review.value(QStringLiteral("karma_up")).toInt();
-                const int usefulTotal = review.value(QStringLiteral("karma_down")).toInt() + usefulFavorable;
+                const int usefulNegative = review.value(QStringLiteral("karma_down")).toInt();
+                const int usefulTotal = usefulFavorable + usefulNegative;
+
+                qreal usefulWilson = 0.f;
+
+                /* from http://www.evanmiller.org/how-not-to-sort-by-average-rating.html */
+                if (usefulFavorable > 0 || usefulNegative > 0) {
+                    usefulWilson = ((usefulFavorable + 1.9208) / (usefulFavorable + usefulNegative)
+                                    - 1.96 * sqrt((usefulFavorable * usefulNegative) / qreal(usefulFavorable + usefulNegative) + 0.9604)
+                                        / (usefulFavorable + usefulNegative))
+                        / (1 + 3.8416 / (usefulFavorable + usefulNegative));
+                    usefulWilson *= 100.f;
+                }
+
                 QDateTime dateTime;
                 dateTime.setSecsSinceEpoch(review.value(QStringLiteral("date_created")).toInt());
+
+                // If there is no score or the score is the same, base on the age
+                QDateTime currentDateTime = QDateTime::currentDateTime();
+                qreal totalDays = dateTime.daysTo(currentDateTime);
+
+                // use also the longest common subsequence of the version string to compute relevance
+                const QString reviewVersion = review.value(QStringLiteral("version")).toString();
+                const QString availableVersion = resource->availableVersion();
+                qreal versionScore = 0;
+                const int minLength = std::min(reviewVersion.length(), availableVersion.length());
+                if (minLength > 0) {
+                    for (int i = 0; i < minLength; ++i) {
+                        if (reviewVersion[i] != availableVersion[i] || i == minLength - 1) {
+                            versionScore = i;
+                            break;
+                        }
+                    }
+                    // Normalize
+                    versionScore = versionScore / qreal(std::max(reviewVersion.length(), availableVersion.length()) - 1);
+                }
+
+                // Very random heuristic which weights usefulness with age and versioin similarity. Don't penalise usefulness more than 6 months
+                usefulWilson = versionScore + 1.0 / std::max(1.0, totalDays) + usefulWilson / std::clamp(totalDays, 1.0, 93.0);
+
                 ReviewPtr r(new Review(review.value(QStringLiteral("app_id")).toString(),
                                        resource->packageName(),
                                        review.value(QStringLiteral("locale")).toString(),
@@ -367,12 +403,13 @@ void OdrsReviewsBackend::parseReviews(const QJsonDocument &document, AbstractRes
                                        review.value(QStringLiteral("description")).toString(),
                                        review.value(QStringLiteral("user_display")).toString(),
                                        dateTime,
-                                       true,
+                                       usefulFavorable >= usefulNegative * 2 || review.value(QStringLiteral("reported")).toInt() > 4,
                                        review.value(QStringLiteral("review_id")).toInt(),
                                        review.value(QStringLiteral("rating")).toInt() / 10,
                                        usefulTotal,
                                        usefulFavorable,
-                                       review.value(QStringLiteral("version")).toString()));
+                                       usefulWilson,
+                                       reviewVersion));
                 // We can also receive just a json with app name and user info so filter these out as there is no review
                 if (!r->summary().isEmpty() && !r->reviewText().isEmpty()) {
                     reviewList << r;
