@@ -9,7 +9,6 @@
 #include <ApplicationAddonsModel.h>
 #include <Category/CategoryModel.h>
 #include <QAbstractItemModelTester>
-#include <QTest>
 #include <ReviewsBackend/ReviewsModel.h>
 #include <ScreenshotsModel.h>
 #include <Transaction/TransactionModel.h>
@@ -19,7 +18,8 @@
 #include <resources/ResourcesProxyModel.h>
 #include <resources/ResourcesUpdatesModel.h>
 
-#include <QtTest>
+#include <QSignalSpy>
+#include <QTest>
 
 QTEST_MAIN(DummyTest)
 
@@ -54,10 +54,10 @@ void DummyTest::initTestCase()
     }
 }
 
-QVector<AbstractResource *> fetchResources(ResultsStream *stream)
+QVector<StreamResult> fetchResources(ResultsStream *stream)
 {
-    QVector<AbstractResource *> ret;
-    QObject::connect(stream, &ResultsStream::resourcesFound, stream, [&ret](const QVector<AbstractResource *> &res) {
+    QVector<StreamResult> ret;
+    QObject::connect(stream, &ResultsStream::resourcesFound, stream, [&ret](const QVector<StreamResult> &res) {
         ret += res;
     });
     QSignalSpy spy(stream, &ResultsStream::destroyed);
@@ -71,8 +71,8 @@ void DummyTest::testReadData()
 
     QCOMPARE(m_appBackend->property("startElements").toInt() * 2, resources.size());
     QBENCHMARK {
-        for (AbstractResource *res : resources) {
-            QVERIFY(!res->name().isEmpty());
+        for (StreamResult res : resources) {
+            QVERIFY(!res.resource->name().isEmpty());
         }
     }
 }
@@ -125,7 +125,8 @@ void DummyTest::testProxySorting()
         const QModelIndex mi = pm.index(i, 0);
 
         const auto value = mi.data(pm.sortRole());
-        QVERIFY(i == 0 || value <= lastRatingCount);
+        QVERIFY(i == 0 || QVariant::compare(value, lastRatingCount) == QPartialOrdering::Less
+                || QVariant::compare(value, lastRatingCount) == QPartialOrdering::Equivalent);
         lastRatingCount = value;
     }
 }
@@ -180,7 +181,7 @@ void DummyTest::testInstallAddons()
 
     const auto resources = fetchResources(m_appBackend->search(filter));
     QCOMPARE(resources.count(), 1);
-    AbstractResource *res = resources.first();
+    AbstractResource *res = resources.first().resource;
     QVERIFY(res);
 
     ApplicationAddonsModel m;
@@ -220,7 +221,7 @@ void DummyTest::testReviewsModel()
 
     const auto resources = fetchResources(m_appBackend->search(filter));
     QCOMPARE(resources.count(), 1);
-    AbstractResource *res = resources.first();
+    AbstractResource *res = resources.first().resource;
     QVERIFY(res);
 
     ReviewsModel m;
@@ -238,7 +239,7 @@ void DummyTest::testReviewsModel()
 
     const auto resources2 = fetchResources(m_appBackend->search(filter));
     QCOMPARE(resources2.count(), 1);
-    res = resources2.first();
+    res = resources2.first().resource;
     m.setResource(res);
     m.fetchMore();
 
@@ -270,7 +271,7 @@ void DummyTest::testScreenshotsModel()
 
     const auto resources = fetchResources(m_appBackend->search(filter));
     QCOMPARE(resources.count(), 1);
-    AbstractResource *res = resources.first();
+    AbstractResource *res = resources.first().resource;
     QVERIFY(res);
     m.setResource(res);
     QCOMPARE(res, m.resource());
