@@ -8,7 +8,7 @@
 #include "KNSBackend.h"
 #include "KNSResource.h"
 #include <KLocalizedString>
-#include <KNSCore/Engine>
+#include <KNSCore/EngineBase>
 #include <KPasswordDialog>
 #include <QDebug>
 #include <QDesktopServices>
@@ -78,6 +78,7 @@ static QVector<ReviewPtr> createReviewList(AbstractResource *app, const Attica::
                                comment.score() / 10,
                                0,
                                0,
+                               0.0,
                                QString()));
         r->addMetadata(QStringLiteral("NumberOfParents"), depth);
         reviews += r;
@@ -116,6 +117,7 @@ void KNSReviews::deleteReview(Review * /*r*/)
 
 void KNSReviews::sendReview(AbstractResource *app, const QString &summary, const QString &review_text, const QString &rating, const QString &userName)
 {
+    Q_UNUSED(userName);
     provider().voteForContent(app->packageName(), rating.toUInt() * 20);
     if (!summary.isEmpty())
         provider().addNewComment(Attica::Comment::ContentComment, app->packageName(), QString(), QString(), summary, review_text);
