@@ -42,6 +42,11 @@ static QStringList packageIds(const QVector<AbstractResource *> &res, std::funct
     return ret;
 }
 
+bool PKTransaction::isLocal() const
+{
+    return m_apps.size() == 1 && qobject_cast<LocalFilePKResource *>(m_apps.at(0));
+}
+
 void PKTransaction::start()
 {
     trigger(PackageKit::Transaction::TransactionFlagSimulate);
@@ -53,15 +58,9 @@ void PKTransaction::trigger(PackageKit::Transaction::TransactionFlags flags)
         m_trans->deleteLater();
     m_newPackageStates.clear();
 
-    if (m_apps.size() == 1 && qobject_cast<LocalFilePKResource *>(m_apps.at(0))) {
+    if (isLocal() && role() == Transaction::InstallRole) {
         auto app = qobject_cast<LocalFilePKResource *>(m_apps.at(0));
         m_trans = PackageKit::Daemon::installFile(QUrl(app->packageName()).toLocalFile(), flags);
-        connect(m_trans.data(), &PackageKit::Transaction::finished, this, [this, app](PackageKit::Transaction::Exit status) {
-            const bool simulate = m_trans->transactionFlags() & PackageKit::Transaction::TransactionFlagSimulate;
-            if (!simulate && status == PackageKit::Transaction::ExitSuccess) {
-                app->setState(AbstractResource::Installed);
-            }
-        });
     } else
         switch (role()) {
         case Transaction::ChangeAddonsRole:
@@ -83,12 +82,17 @@ void PKTransaction::trigger(PackageKit::Transaction::TransactionFlags flags)
         } break;
         case Transaction::RemoveRole:
             // see bug #315063
+#ifdef PACKAGEKIT_AUTOREMOVE
+            constexpr bool autoremove = true;
+#else
+            constexpr bool autoremove = false;
+#endif
             m_trans = PackageKit::Daemon::removePackages(packageIds(m_apps,
                                                                     [](PackageKitResource *r) {
                                                                         return r->installedPackageId();
                                                                     }),
                                                          true /*allowDeps*/,
-                                                         false,
+                                                         autoremove,
                                                          flags);
             break;
         };
@@ -126,7 +130,7 @@ void PKTransaction::progressChanged()
 {
     auto percent = m_trans->percentage();
     if (percent == 101) {
-        qWarning() << "percentage cannot be calculated";
+        qCWarning(LIBDISCOVER_BACKEND_LOG) << "percentage cannot be calculated";
         percent = 50;
     }
 
@@ -177,7 +181,9 @@ void PKTransaction::cleanup(PackageKit::Transaction::Exit exit, uint runtime)
         }
         removedResources.subtract(kToSet(m_apps));
 
-        auto isCritical = [](AbstractResource *r) { return static_cast<PackageKitResource *>(r)->isCritical(); };
+        auto isCritical = [](AbstractResource *r) {
+            return static_cast<PackageKitResource *>(r)->isCritical();
+        };
         auto criticals = kFilter<QSet<AbstractResource *>>(removedResources, isCritical);
         criticals.unite(kFilter<QSet<AbstractResource *>>(m_apps, isCritical));
         auto resourceName = [](AbstractResource *a) {
@@ -210,6 +216,8 @@ void PKTransaction::cleanup(PackageKit::Transaction::Exit exit, uint runtime)
     }
 
     this->submitResolve();
+    if (isLocal())
+        qobject_cast<LocalFilePKResource *>(m_apps.at(0))->resolve({});
     if (failed)
         setStatus(Transaction::DoneWithErrorStatus);
     else if (cancel)
@@ -241,7 +249,7 @@ void PKTransaction::proceed()
     if (!m_proceedFunctions.isEmpty()) {
         processProceedFunction();
     } else {
-        if (m_apps.size() == 1 && qobject_cast<LocalFilePKResource *>(m_apps.at(0))) {
+        if (isLocal()) {
             trigger(PackageKit::Transaction::TransactionFlagNone);
         } else {
             trigger(PackageKit::Transaction::TransactionFlagOnlyTrusted);
