@@ -74,7 +74,7 @@ bool AbstractResource::isInstalled()
 
 void AbstractResource::fetchScreenshots()
 {
-    Q_EMIT screenshotsFetched({}, {});
+    Q_EMIT screenshotsFetched({});
 }
 
 QStringList AbstractResource::mimetypes() const
@@ -271,15 +271,9 @@ QString AbstractResource::versionString()
     const QString version = isInstalled() ? installedVersion() : availableVersion();
     if (version.isEmpty()) {
         return {};
-    } else {
-        QLocale l;
-        const QString releaseString = l.toString(releaseDate(), QLocale::ShortFormat);
-        if (!releaseString.isEmpty()) {
-            return i18n("%1, released on %2", version, releaseString);
-        } else {
-            return version;
-        }
     }
+
+    return version;
 }
 
 QString AbstractResource::contentRatingDescription() const
