@@ -268,8 +268,8 @@ void ResourcesProxyModel::addResources(const QVector<StreamResult> &_res)
     if (res.isEmpty())
         return;
 
-    std::sort(res.begin(), res.end(), [this](auto res, auto res2) {
-        return orderedLessThan(res, res2);
+    std::sort(res.begin(), res.end(), [this](const auto &left, const auto &right) {
+        return orderedLessThan(left, right);
     });
 
     sortedInsertion(res);
@@ -282,8 +282,8 @@ void ResourcesProxyModel::invalidateSorting()
         return;
 
     beginResetModel();
-    std::sort(m_displayedResources.begin(), m_displayedResources.end(), [this](auto res, auto res2) {
-        return orderedLessThan(res, res2);
+    std::sort(m_displayedResources.begin(), m_displayedResources.end(), [this](const auto &left, const auto &right) {
+        return orderedLessThan(left, right);
     });
     endResetModel();
 }
@@ -387,7 +387,7 @@ void ResourcesProxyModel::invalidateFilter()
     }
 
     m_currentStream = m_filters.backend ? m_filters.backend->search(m_filters) : ResourcesModel::global()->search(m_filters);
-    Q_EMIT busyChanged(true);
+    Q_EMIT busyChanged();
 
     if (!m_displayedResources.isEmpty()) {
         beginResetModel();
@@ -398,7 +398,7 @@ void ResourcesProxyModel::invalidateFilter()
     connect(m_currentStream, &ResultsStream::resourcesFound, this, &ResourcesProxyModel::addResources);
     connect(m_currentStream, &ResultsStream::destroyed, this, [this]() {
         m_currentStream = nullptr;
-        Q_EMIT busyChanged(false);
+        Q_EMIT busyChanged();
     });
 }
 
@@ -407,43 +407,38 @@ int ResourcesProxyModel::rowCount(const QModelIndex &parent) const
     return parent.isValid() ? 0 : m_displayedResources.count();
 }
 
+// This comparator takes m_sortRole and m_sortOrder into account. It has a
+// fallback mechanism to use secondary sort role and order.
 bool ResourcesProxyModel::orderedLessThan(const StreamResult &left, const StreamResult &right) const
 {
-    bool less = lessThan(left, right);
-    return m_sortOrder == Qt::AscendingOrder ? less : !less;
-}
+    // (role, order) pair
+    using SortCombination = std::pair<Roles, Qt::SortOrder>;
+    const std::array<SortCombination, 2> sortFallbackChain = {{
+        {m_sortRole, m_sortOrder},
+        {NameRole, Qt::AscendingOrder},
+    }};
 
-bool ResourcesProxyModel::lessThan(const StreamResult &left, const StreamResult &right) const
-{
-    AbstractResource *leftPackage = left.resource;
-    AbstractResource *rightPackage = right.resource;
-
-    auto role = m_sortRole;
-    Qt::SortOrder order = m_sortOrder;
-    QVariant leftValue;
-    QVariant rightValue;
-    // if we're comparing two equal values, we want the model sorted by application name
-    if (role != NameRole) {
-        leftValue = roleToValue(left, role);
-        rightValue = roleToValue(right, role);
+    for (const auto &[role, order] : sortFallbackChain) {
+        QVariant leftValue = roleToOrderedValue(left, role);
+        QVariant rightValue = roleToOrderedValue(right, role);
 
         if (leftValue == rightValue) {
-            role = NameRole;
-            order = Qt::AscendingOrder;
+            continue;
         }
+
+        const auto result = QVariant::compare(leftValue, rightValue);
+
+        // Should not happen, but it's better to skip than assert
+        if (!(result == QPartialOrdering::Less || result == QPartialOrdering::Greater)) {
+            continue;
+        }
+
+        // Yes, there is a shorter but incomprehensible way of rewriting this
+        return (order == Qt::AscendingOrder) ? (result == QPartialOrdering::Less) : (result == QPartialOrdering::Greater);
     }
 
-    bool ret;
-    if (role == NameRole) {
-        ret = leftPackage->nameSortKey().compare(rightPackage->nameSortKey()) < 0;
-    } else if (role == CanUpgrade) {
-        ret = leftValue.toBool();
-    } else {
-        const auto order = QVariant::compare(leftValue, rightValue);
-        Q_ASSERT(order != QPartialOrdering::Unordered);
-        return order == QPartialOrdering::Less;
-    }
-    return ret != (order != Qt::AscendingOrder);
+    // They compared equal, so it is definitely not a "less than" relation
+    return false;
 }
 
 Category *ResourcesProxyModel::filteredCategory() const
@@ -610,12 +605,24 @@ QVariant ResourcesProxyModel::roleToValue(const StreamResult &result, int role)
     }
 }
 
+// Wraps roleToValue with additional features for sorting/comparison.
+QVariant ResourcesProxyModel::roleToOrderedValue(const StreamResult &result, int role) const
+{
+    AbstractResource *resource = result.resource;
+    switch (role) {
+    case NameRole:
+        return QVariant::fromValue(resource->nameSortKey());
+    default:
+        return roleToValue(result, role);
+    }
+}
+
 bool ResourcesProxyModel::isSorted(const QVector<StreamResult> &resources)
 {
     auto last = resources.constFirst();
     for (auto it = resources.constBegin() + 1, itEnd = resources.constEnd(); it != itEnd; ++it) {
         auto v1 = roleToValue(last, m_sortRole), v2 = roleToValue(*it, m_sortRole);
-        if (!lessThan(last, *it) && v1 != v2) {
+        if (!orderedLessThan(last, *it) && v1 != v2) {
             qDebug() << "faulty sort" << last.resource->name() << (*it).resource->name() << last.resource << (*it).resource;
             return false;
         }
@@ -646,7 +653,7 @@ void ResourcesProxyModel::sortedInsertion(const QVector<StreamResult> &_res)
     }
 
     for (auto result : std::as_const(resources)) {
-        const auto finder = [this](StreamResult result, StreamResult res) {
+        const auto finder = [this](const StreamResult &result, const StreamResult &res) {
             return orderedLessThan(result, res);
         };
         const auto it = std::upper_bound(m_displayedResources.constBegin(), m_displayedResources.constEnd(), result, finder);
