@@ -60,7 +60,7 @@ void StandardBackendUpdater::start()
     });
 
     const bool couldCancel = m_canCancel;
-    for (AbstractResource *res : qAsConst(upgradeList)) {
+    for (AbstractResource *res : std::as_const(upgradeList)) {
         m_pendingResources += res;
         auto t = m_backend->installApplication(res);
         t->setVisible(false);
@@ -132,11 +132,15 @@ void StandardBackendUpdater::transactionRemoved(Transaction *t)
     }
 
     const bool found = fromOurBackend && m_pendingResources.remove(t->resource());
+    m_anyTransactionFailed |= t->status() != Transaction::DoneStatus;
 
     if (found && !m_settingUp) {
         refreshProgress();
         if (m_pendingResources.isEmpty()) {
             cleanup();
+            if (needsReboot() && !m_anyTransactionFailed) {
+                enableReadyToReboot();
+            }
         }
     }
     refreshUpdateable();
@@ -173,10 +177,10 @@ void StandardBackendUpdater::refreshUpdateable()
     f.state = AbstractResource::Upgradeable;
     m_upgradeable.clear();
     auto r = m_backend->search(f);
-    connect(r, &ResultsStream::resourcesFound, this, [this](const QVector<AbstractResource *> &resources) {
+    connect(r, &ResultsStream::resourcesFound, this, [this](const QVector<StreamResult> &resources) {
         for (auto res : resources)
-            if (res->state() == AbstractResource::Upgradeable)
-                m_upgradeable.insert(res);
+            if (res.resource->state() == AbstractResource::Upgradeable)
+                m_upgradeable.insert(res.resource);
     });
     connect(r, &ResultsStream::destroyed, this, [this]() {
         m_settingUp = false;
